DOCS = """
Jenks is a command-line tool to work with a jenkins host and jobs.

Configuration (.jenksrc)
========================

Jenks requires a .jenksrc file to configure itself. jenks searches in
the current directory and moves up parent directories until it finds one.

A .jenksrc file should be a yaml file, with a dictionary of hosts and
some keys for configuration.

Here is an example .jenksrc file with all options:

    'localhost':
      url: 'http://localhost:8080/'
      jobs:
        - foo
        - bar
    'http://jenkins.local':
      jobs:
        - baz

the in-depth configuration for a host is:

* url: the url to the host. If the url doesn't exist, the host
       name is used as the url
* jobs: a list of job names to add to the list

Keys
====

By default, jenks will perform operations on all jobs in the
configuration. However, you can restrict the jobs to specific ones by
passing a list of keys. Each job is given a unique key generated by
jenks. For example, using -l on the example configuration will yield:

    $ jenks -l
    0 http://jenkins.local baz
    1 localhost foo 
    2 localhost bar

The character at the beginning of each line is the key. Each key is a
single character in the set: [0-9a-zA-Z]. You can pass in a string of characters to get information about those:

    $ jenks -l 20
    0 http://jenkins.local baz
    2 localhost bar

You can pass in keys by:

* keys as an argument:

    $ jenks -c 0

* keys with standard in:

    $ echo '0' | jenks

Examples
========

Ones of jenks' goals is to follow the Unix philosophy of doing one
thing and doing it well. By being a modular part and supporting stdin,
it's possible to chain jenks with other commands.

For example, here's a command that only lists status whose host or jobs match a specific regex:

    $ jenks -l | grep "foo" | cut -d ' ' -f 1 | jenks
    0: localhost, foo (last build #7) SUCCESS

Usage
=====

{0}
""".strip()



"""Jenks, a Jenkins command line tool.
Usage:
  jenks [<keys>] [-c | -l | -t]
  jenks (-d | --docs)
  jenks (-h | --help)

Options:
  -c, --console   print console information
  -d, --docs      list full documentation
  -l, --list      list the jobs
  -t, --trigger   trigger jobs
  -h, --help      print this help guide. use `jenks -d` for a full page of documentation
"""
import signal
import sys

from docopt import docopt
from .data import get_configuration, JenksData
from .command import List, Console, Status, Trigger
from .docs import DOCS


def signal_handler(signal, frame):
    sys.exit(0)

DEFAULT_COMMAND = Status
ARGUMENT_COMMANDS = [List, Console, Trigger]


def main(argv=sys.argv[1:]):
    signal.signal(signal.SIGINT, signal_handler)
    options = docopt(__doc__, argv=argv, version="jenks 0.1")
    if options['--docs']:
        print(DOCS.format(__doc__))
        sys.exit(0)
    try:
        data = JenksData(get_configuration())
        inp = None
        if not sys.stdin.isatty():
            inp = sys.stdin.read().strip().replace('\n', '')
        keys = options['<keys>'] or inp or data.job_keys()
        command = DEFAULT_COMMAND
        for cmd in ARGUMENT_COMMANDS:
            if options[cmd.argument]:
                command = cmd
        for job in data.jobs(sorted(keys)):
            print(command.act(job))
    except Exception as e:
        print("Error! {0}".format(str(e)))

if __name__ == '__main__':
    main()



import os
from jenkinsapi.jenkins import Jenkins
from collections import namedtuple
import yaml

CONFIG_FILE_NAME = ".jenksrc"

STATUS_STRING = "{key}: {host}, {name} (build #{number}) {status}"
LIST_TEMPLATE = "{key} {host} {name}"

KEYS = ('0', '1', '2', '3', '4')

UNABLE_TO_FIND_JENKS_CONFIG = """
Unable to find {0} file! Maybe you need to add one?

Jenks searches for files from the current working directory up.

Example .jenksrc:

localhost:
  url: 'http://localhost:8080/'
  jobs:
    - foo
    - bar
""".format(CONFIG_FILE_NAME)


class JenksDataException(Exception):
    """ Wrapper for configs """

JenksJob = namedtuple('JenksJob', 'key, host, name, api_instance')


class JenksData(object):

    def __init__(self, config_dict):
        self._parse_dict(config_dict)

    def job_keys(self):
        """ return a list of jobs """
        return (j for j in self._jobs)

    def jobs(self, job_keys):
        return (self._jobs[key] for key in job_keys)

    def _parse_dict(self, config_dict):
        """ parse the dictionary into a config object """
        self.hosts_info = config_dict
        self.hosts = {}
        self._jobs = {}
        self._empty_key_index = 0
        for host in sorted(config_dict.keys()):
            info = config_dict[host]
            url = info.get('url', host)
            self.hosts[host] = Jenkins(url)
            if 'jobs' in info:
                for job_name in info['jobs']:
                    self._add_job(host, job_name)

    def _add_job(self, host, job_name):
        key = KEYS[self._empty_key_index]
        value = JenksJob(key, host, job_name, self.hosts[host][job_name])
        self._jobs[key] = value
        self._empty_key_index += 1


def get_configuration():
    """ return jenks configuration """
    path = os.path.abspath(os.curdir)
    while path != os.sep:
        config_path = os.path.join(path, CONFIG_FILE_NAME)
        if os.path.exists(config_path):
            with open(config_path) as fh:
                return yaml.load(fh.read())
        path = os.path.dirname(path)
    raise JenksDataException(UNABLE_TO_FIND_JENKS_CONFIG)



"""
A file to store jenks command classes
"""
import requests


class Status(object):
    """ Return the status of jobs """

    argument = None
    TEMPLATE = "{key}: {host}, {name} (last build #{number}) {status}"

    @staticmethod
    def act(job):
        last_build = job.api_instance.get_last_build()

        if job.api_instance.is_queued():
            status = "queued"
        else:
            status = last_build.get_status() or "running..."

        return Status.TEMPLATE.format(
            key=job.key,
            host=job.host,
            name=job.name,
            number=last_build.get_number(),
            status=status
        )


class Console(object):
    """ return the console output of jobs """

    argument = "--console"

    @staticmethod
    def act(job):
        return job.api_instance.get_last_build().get_console()


class List(object):
    """ list the information about the job without retrieving information online """

    argument = "--list"
    TEMPLATE = "{key} {host} {name}"

    @staticmethod
    def act(job):
        return List.TEMPLATE.format(
            key=job.key,
            host=job.host,
            name=job.name
        )


class Trigger(object):
    """" trigger jobs """

    argument = "--trigger"

    @staticmethod
    def act(job):
        if job.api_instance.is_running():
            return "{0}, {1} is already running".format(job.host, job.name)
        else:
            requests.get(job.api_instance.get_build_triggerurl())
            return "triggering {0}, {1}...".format(job.host, job.name)



#!/usr/bin/env python

try:
    from setuptools import setup, find_packages
except:
    from distutils.core import setup

setup(name='jenks',
      version='0.2',
      description='a jenkins command-line tool',
      long_description=open('README.rst').read(),
      author='Yusuke Tsutsumi',
      author_email='yusuke@yusuketsutsumi.com',
      url='http://toumorokoshi.github.io/jenks',
      packages=find_packages(),
      install_requires=[
          'requests>=2.3.0',
          'docopt>=0.6.1',
          'jenkinsapi>=0.2.18',
          'pyyaml>=3.10'
      ],
      classifiers=[
          'Development Status :: 2 - Pre-Alpha',
          'Topic :: System :: Software Distribution',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7'
      ],
      entry_points={
          'console_scripts': [
              'jenks = jenks:main'
          ]
      },
      tests_require=['mock>=1.0.1', 'nose>=1.3.0', 'httpretty==0.6.5'],
      test_suite='nose.collector'
)
